@@ -19,7 +19,6 @@ from Orange.widgets import widget, settings, gui
 from Orange.widgets.gui import HorizontalGridDelegate
 from Orange.widgets.utils import itemmodels, colorpalettes
 from Orange.widgets.utils.widgetpreview import WidgetPreview
-from Orange.widgets.utils.state_summary import format_summary_details
 from Orange.widgets.report import colored_square as square
 from Orange.widgets.widget import Input, Output
 
@@ -580,9 +579,6 @@ class OWColor(widget.OWWidget):
         gui.rubber(self.buttonsArea)
         gui.auto_apply(self.buttonsArea, self, "auto_apply")
 
-        self.info.set_input_summary(self.info.NoInput)
-        self.info.set_output_summary(self.info.NoOutput)
-
     @staticmethod
     def sizeHint():  # pragma: no cover
         return QSize(500, 570)
@@ -594,10 +590,8 @@ class OWColor(widget.OWWidget):
         self.cont_descs = []
         if data is None:
             self.data = self.domain = None
-            self.info.set_input_summary(self.info.NoInput)
         else:
             self.data = data
-            self.info.set_input_summary(len(data), format_summary_details(data))
             for var in chain(data.domain.variables, data.domain.metas):
                 if var.is_discrete:
                     self.disc_descs.append(DiscAttrDesc(var))
@@ -609,15 +603,17 @@ class OWColor(widget.OWWidget):
         self.openContext(data)
         self.disc_view.resizeColumnsToContents()
         self.cont_view.resizeColumnsToContents()
-        self.unconditional_commit()
+        self.commit.now()
 
     def _on_data_changed(self):
-        self.commit()
+        self.commit.deferred()
 
     def reset(self):
         self.disc_model.reset()
         self.cont_model.reset()
-        self.commit()
+        # Reset button is in the same box as Load, which has commit.now,
+        # and Apply, hence let Reset commit now, too.
+        self.commit.now()
 
     def save(self):
         fname, _ = QFileDialog.getSaveFileName(
@@ -651,16 +647,15 @@ class OWColor(widget.OWWidget):
             return
 
         try:
-            f = open(fname)
+            with open(fname) as f:
+                js = json.load(f)  #: dict
+                self._parse_var_defs(js)
         except IOError:
             QMessageBox.critical(self, "File error", "File cannot be opened.")
             return
-
-        try:
-            js = json.load(f)  #: dict
-            self._parse_var_defs(js)
         except (json.JSONDecodeError, InvalidFileFormat):
             QMessageBox.critical(self, "File error", "Invalid file format.")
+            return
 
     def _parse_var_defs(self, js):
         if not isinstance(js, dict) or set(js) != {"categorical", "numeric"}:
@@ -694,6 +689,7 @@ class OWColor(widget.OWWidget):
 
         # First, construct all descriptions; assign later, after we know
         # there won't be exceptions due to invalid file format
+        unused_vars = []
         both_descs = []
         warnings = []
         for old_desc, repo, desc_type in (
@@ -704,11 +700,26 @@ class OWColor(widget.OWWidget):
             for var_name, var_data in js[repo].items():
                 var = var_by_name.get(var_name)
                 if var is None:
+                    unused_vars.append(var_name)
                     continue
                 # This can throw InvalidFileFormat
                 new_descs[var_name], warn = desc_type.from_dict(var, var_data)
                 warnings += warn
             both_descs.append(new_descs)
+        if unused_vars:
+            names = [f"'{name}'" for name in unused_vars]
+            if len(unused_vars) == 1:
+                warn = f'Definition for variable {names[0]}, which does not ' \
+                       f'appear in the data, was ignored.\n'
+            else:
+                if len(unused_vars) <= 5:
+                    warn = 'Definitions for variables ' \
+                           f'{", ".join(names[:-1])} and {names[-1]}'
+                else:
+                    warn = f'Definitions for {", ".join(names[:4])} ' \
+                           f'and {len(names) - 4} other variables'
+                warn += ", which do not appear in the data, were ignored.\n"
+            warnings.insert(0, warn)
 
         self.disc_descs = [both_descs[0].get(desc.var.name, desc)
                            for desc in self.disc_descs]
@@ -720,13 +731,14 @@ class OWColor(widget.OWWidget):
 
         self.disc_model.set_data(self.disc_descs)
         self.cont_model.set_data(self.cont_descs)
-        self.unconditional_commit()
+        self.commit.now()
 
     def _start_dir(self):
         return self.workflowEnv().get("basedir") \
                or QSettings().value("colorwidget/last-location") \
                or os.path.expanduser(f"~{os.sep}")
 
+    @gui.deferred
     def commit(self):
         def make(variables):
             new_vars = []
@@ -738,7 +750,6 @@ class OWColor(widget.OWWidget):
 
         if self.data is None:
             self.Outputs.data.send(None)
-            self.info.set_output_summary(self.info.NoOutput)
             return
 
         disc_dict = {desc.var.name: desc for desc in self.disc_descs}
@@ -748,8 +759,6 @@ class OWColor(widget.OWWidget):
         new_domain = Orange.data.Domain(
             make(dom.attributes), make(dom.class_vars), make(dom.metas))
         new_data = self.data.transform(new_domain)
-        self.info.set_output_summary(len(new_data),
-                                     format_summary_details(new_data))
         self.Outputs.data.send(new_data)
 
     def send_report(self):
