@@ -15,7 +15,7 @@ import numpy as np
 import scipy.stats as ss
 import scipy.sparse as sp
 from AnyQt.QtCore import Qt, QSize, QRectF, QModelIndex, pyqtSlot, \
-    QRegExp, QItemSelection, QItemSelectionRange, QItemSelectionModel
+    QItemSelection, QItemSelectionRange, QItemSelectionModel
 from AnyQt.QtGui import QPainter, QColor
 from AnyQt.QtWidgets import QStyledItemDelegate, QGraphicsScene, QTableView, \
     QHeaderView, QStyle, QStyleOptionViewItem
@@ -23,6 +23,7 @@ from AnyQt.QtWidgets import QStyledItemDelegate, QGraphicsScene, QTableView, \
 import Orange.statistics.util as ut
 from Orange.data import Table, StringVariable, DiscreteVariable, \
     ContinuousVariable, TimeVariable, Domain, Variable
+from Orange.util import utc_from_timestamp
 from Orange.widgets import widget, gui
 from Orange.widgets.data.utils.histogram import Histogram
 from Orange.widgets.settings import Setting, ContextSetting, \
@@ -30,7 +31,6 @@ from Orange.widgets.settings import Setting, ContextSetting, \
 from Orange.widgets.utils.itemmodels import DomainModel, AbstractSortTableModel
 from Orange.widgets.utils.signals import Input, Output
 from Orange.widgets.utils.widgetpreview import WidgetPreview
-from Orange.widgets.utils.state_summary import format_summary_details
 
 
 def _categorical_entropy(x):
@@ -69,8 +69,8 @@ def format_time_diff(start, end, round_up_after=2):
     str
 
     """
-    start = datetime.datetime.fromtimestamp(start)
-    end = datetime.datetime.fromtimestamp(end)
+    start = utc_from_timestamp(start)
+    end = utc_from_timestamp(end)
     diff = abs(end - start)  # type: datetime.timedelta
 
     # Get the different resolutions
@@ -110,14 +110,16 @@ class FeatureStatisticsTableModel(AbstractSortTableModel):
     HIDDEN_VAR_TYPES = (StringVariable,)
 
     class Columns(IntEnum):
-        ICON, NAME, DISTRIBUTION, CENTER, DISPERSION, MIN, MAX, MISSING = range(8)
+        ICON, NAME, DISTRIBUTION, CENTER, MEDIAN, DISPERSION, MIN, MAX, \
+        MISSING = range(9)
 
         @property
         def name(self):
             return {self.ICON: '',
                     self.NAME: 'Name',
                     self.DISTRIBUTION: 'Distribution',
-                    self.CENTER: 'Center',
+                    self.CENTER: 'Mean',
+                    self.MEDIAN: 'Median',
                     self.DISPERSION: 'Dispersion',
                     self.MIN: 'Min.',
                     self.MAX: 'Max.',
@@ -157,7 +159,7 @@ class FeatureStatisticsTableModel(AbstractSortTableModel):
 
         no_data = np.array([])
         self._variable_types = self._variable_names = no_data
-        self._min = self._max = self._center = no_data
+        self._min = self._max = self._center = self._median = no_data
         self._dispersion = no_data
         self._missing = no_data
         # Clear model initially to set default values
@@ -175,11 +177,12 @@ class FeatureStatisticsTableModel(AbstractSortTableModel):
         self.domain = domain = data.domain
         self.target_var = None
 
-        self.__attributes = self.__filter_attributes(domain.attributes, self.table.X)
-        # We disable pylint warning because the `Y` property squeezes vectors,
-        # while we need a 2d array, which `_Y` provides
-        self.__class_vars = self.__filter_attributes(domain.class_vars, self.table._Y)  # pylint: disable=protected-access
-        self.__metas = self.__filter_attributes(domain.metas, self.table.metas)
+        self.__attributes = self.__filter_attributes(
+            domain.attributes, self.table.X)
+        self.__class_vars = self.__filter_attributes(
+            domain.class_vars, self.table.Y.reshape((len(self.table.Y), -1)))
+        self.__metas = self.__filter_attributes(
+            domain.metas, self.table.metas)
         self.__attributes_set = set(self.__metas[0])
         self.__class_vars_set = set(self.__class_vars[0])
         self.__metas_set = set(self.__metas[0])
@@ -275,11 +278,18 @@ class FeatureStatisticsTableModel(AbstractSortTableModel):
 
         self._center = self.__compute_stat(
             matrices,
-            discrete_f=lambda x: __mode(x, axis=0),
+            discrete_f=None,
             continuous_f=lambda x: ut.nanmean(x, axis=0),
             time_f=lambda x: ut.nanmean(x, axis=0),
         )
 
+        self._median = self.__compute_stat(
+            matrices,
+            discrete_f=lambda x: __mode(x, axis=0),
+            continuous_f=lambda x: ut.nanmedian(x, axis=0),
+            time_f=lambda x: ut.nanmedian(x, axis=0),
+        )
+
     def get_statistics_matrix(self, variables=None, return_labels=False):
         """Get the numeric computed statistics in a single matrix. Optionally,
         we can specify for which variables we want the stats. Also, we can get
@@ -311,14 +321,16 @@ class FeatureStatisticsTableModel(AbstractSortTableModel):
             indices = ...
 
         matrix = np.vstack((
-            self._center[indices], self._dispersion[indices],
+            self._center[indices], self._median[indices],
+            self._dispersion[indices],
             self._min[indices], self._max[indices], self._missing[indices],
         )).T
 
         # Return string labels for the returned matrix columns e.g. 'Mean',
         # 'Dispersion' if requested
         if return_labels:
-            labels = [self.Columns.CENTER.name, self.Columns.DISPERSION.name,
+            labels = [self.Columns.CENTER.name, self.Columns.MEDIAN.name,
+                      self.Columns.DISPERSION.name,
                       self.Columns.MIN.name, self.Columns.MAX.name,
                       self.Columns.MISSING.name]
             return labels, matrix
@@ -375,14 +387,17 @@ class FeatureStatisticsTableModel(AbstractSortTableModel):
         to group those variable types together.
         """
         # Prepare indices for variable types so we can group them together
-        order = [DiscreteVariable, ContinuousVariable, TimeVariable, StringVariable]
+        order = [ContinuousVariable, TimeVariable,
+                 DiscreteVariable, StringVariable]
         mapping = {var: idx for idx, var in enumerate(order)}
         vmapping = np.vectorize(mapping.__getitem__)
         var_types_indices = vmapping(self._variable_types)
 
         # Store the variable name sorted indices so we can pass a default
         # order when sorting by multiple keys
-        var_name_indices = np.argsort(self._variable_names)
+        # Double argsort is "inverse" argsort:
+        # data will be *sorted* by these indices
+        var_name_indices = np.argsort(np.argsort(self._variable_names))
 
         # Prepare vartype indices so ready when needed
         disc_idx, _, time_idx, str_idx = self._attr_indices(self.variables)
@@ -406,6 +421,13 @@ class FeatureStatisticsTableModel(AbstractSortTableModel):
             vals[disc_idx] = var_name_indices[disc_idx]
             vals[str_idx] = var_name_indices[str_idx]
             return np.vstack((var_types_indices, np.zeros_like(vals), vals)).T
+        # Sort by: (type, median)
+        elif column == self.Columns.MEDIAN:
+            # Sorting discrete or string values by median makes no sense
+            vals = np.array(self._median)
+            vals[disc_idx] = var_name_indices[disc_idx]
+            vals[str_idx] = var_name_indices[str_idx]
+            return np.vstack((var_types_indices, np.zeros_like(vals), vals)).T
         # Sort by: (type, dispersion)
         elif column == self.Columns.DISPERSION:
             # Sort time variables by their dispersion, which is not stored in
@@ -443,12 +465,21 @@ class FeatureStatisticsTableModel(AbstractSortTableModel):
 
     def _argsortData(self, data, order):
         if data.ndim == 1:
-            indices = np.argsort(data, kind='mergesort')
-            if order == Qt.DescendingOrder:
-                indices = indices[::-1]
-            # Always sort NaNs last
             if np.issubdtype(data.dtype, np.number):
-                indices = np.roll(indices, -np.isnan(data).sum())
+                if order == Qt.DescendingOrder:
+                    data = -data
+                indices = np.argsort(data, kind='stable')
+                # Always sort NaNs last
+                if np.issubdtype(data.dtype, np.number):
+                    indices = np.roll(indices, -np.isnan(data).sum())
+            else:
+                # When not sorting by numbers, we can't do data = -data, but
+                # use indices = indices[::-1] instead. This is not stable, but
+                # doesn't matter because we use this only for variable names
+                # which are guaranteed to be unique
+                indices = np.argsort(data)
+                if order == Qt.DescendingOrder:
+                    indices = indices[::-1]
         else:
             assert np.issubdtype(data.dtype, np.number), \
                 'We do not deal with non numeric values in sorting by ' \
@@ -531,6 +562,7 @@ class FeatureStatisticsTableModel(AbstractSortTableModel):
                             variable=attribute,
                             color_attribute=self.target_var,
                             border=(0, 0, 2, 0),
+                            bottom_padding=4,
                             border_color='#ccc',
                         )
                         scene.addItem(histogram)
@@ -538,6 +570,8 @@ class FeatureStatisticsTableModel(AbstractSortTableModel):
                     return self.__distributions_cache[row]
             elif column == self.Columns.CENTER:
                 return render_value(self._center[row])
+            elif column == self.Columns.MEDIAN:
+                return render_value(self._median[row])
             elif column == self.Columns.DISPERSION:
                 if isinstance(attribute, TimeVariable):
                     return format_time_diff(self._min[row], self._max[row])
@@ -647,7 +681,7 @@ class FeatureStatisticsTableView(QTableView):
         if logical_index is not self.model().Columns.DISTRIBUTION.index:
             return
         ratio_width, ratio_height = self.HISTOGRAM_ASPECT_RATIO
-        unit_width = new_size / ratio_width
+        unit_width = new_size // ratio_width
         new_height = unit_width * ratio_height
         effective_height = max(new_height, self.MINIMUM_HISTOGRAM_HEIGHT)
         effective_height = min(effective_height, self.MAXIMUM_HISTOGRAM_HEIGHT)
@@ -676,7 +710,7 @@ class NoFocusRectDelegate(QStyledItemDelegate):
         super().paint(painter, option, index)
 
 
-class DistributionDelegate(QStyledItemDelegate):
+class DistributionDelegate(NoFocusRectDelegate):
     def paint(self, painter, option, index):
         # type: (QPainter, QStyleOptionViewItem, QModelIndex) -> None
         scene = index.data(Qt.DisplayRole)  # type: Optional[QGraphicsScene]
@@ -684,14 +718,6 @@ class DistributionDelegate(QStyledItemDelegate):
             return super().paint(painter, option, index)
 
         painter.setRenderHint(QPainter.Antialiasing)
-
-        if option.state & QStyle.State_Selected:
-            background_color = option.palette.highlight()
-        else:
-            background_color = index.data(Qt.BackgroundRole)
-        if background_color is not None:
-            scene.setBackgroundBrush(background_color)
-
         scene.render(painter, target=QRectF(option.rect), mode=Qt.IgnoreAspectRatio)
 
         # pylint complains about inconsistent return statements
@@ -719,7 +745,7 @@ class OWFeatureStatistics(widget.OWWidget):
     color_var = ContextSetting(None)  # type: Optional[Variable]
     # filter_string = ContextSetting('')
 
-    sorting = Setting((0, Qt.DescendingOrder))
+    sorting = Setting((0, Qt.AscendingOrder))
     selected_vars = ContextSetting([], schema_only=True)
 
     def __init__(self):
@@ -727,16 +753,6 @@ class OWFeatureStatistics(widget.OWWidget):
 
         self.data = None  # type: Optional[Table]
 
-        # TODO: Implement filtering on the model
-        # filter_box = gui.vBox(self.controlArea, 'Filter')
-        # self.filter_text = gui.lineEdit(
-        #     filter_box, self, value='filter_string',
-        #     placeholderText='Filter variables by name',
-        #     callback=self._filter_table_variables, callbackOnType=True,
-        # )
-        # shortcut = QShortcut(QKeySequence('Ctrl+f'), self, self.filter_text.setFocus)
-        # shortcut.setWhatsThis('Filter variables by name')
-
         # Main area
         self.model = FeatureStatisticsTableModel(parent=self)
         self.table_view = FeatureStatisticsTableView(self.model, parent=self)
@@ -759,24 +775,10 @@ class OWFeatureStatistics(widget.OWWidget):
         gui.rubber(self.buttonsArea)
         gui.auto_send(self.buttonsArea, self, "auto_commit")
 
-        self.info.set_input_summary(self.info.NoInput)
-        self.info.set_output_summary(self.info.NoOutput)
-
     @staticmethod
     def sizeHint():
         return QSize(1050, 500)
 
-    def _filter_table_variables(self):
-        regex = QRegExp(self.filter_string)
-        # If the user explicitly types different cases, we assume they know
-        # what they are searching for and account for letter case in filter
-        different_case = (
-            any(c.islower() for c in self.filter_string) and
-            any(c.isupper() for c in self.filter_string)
-        )
-        if not different_case:
-            regex.setCaseSensitivity(Qt.CaseInsensitive)
-
     @Inputs.data
     def set_data(self, data):
         # Clear outputs and reset widget state
@@ -790,14 +792,11 @@ class OWFeatureStatistics(widget.OWWidget):
         self.data = data
 
         if data is not None:
-            self.info.set_input_summary(len(data),
-                                        format_summary_details(data))
             self.color_var_model.set_domain(data.domain)
             self.color_var = None
             if self.data.domain.class_vars:
                 self.color_var = self.data.domain.class_vars[0]
         else:
-            self.info.set_input_summary(self.info.NoInput)
             self.color_var_model.set_domain(None)
             self.color_var = None
         self.model.set_data(data)
@@ -805,7 +804,6 @@ class OWFeatureStatistics(widget.OWWidget):
         self.openContext(self.data)
         self.__restore_selection()
         self.__restore_sorting()
-        # self._filter_table_variables()
         self.__color_var_changed()
 
         self.commit()
@@ -852,15 +850,12 @@ class OWFeatureStatistics(widget.OWWidget):
 
     def commit(self):
         if not self.selected_vars:
-            self.info.set_output_summary(self.info.NoOutput)
             self.Outputs.reduced_data.send(None)
             self.Outputs.statistics.send(None)
             return
 
         # Send a table with only selected columns to output
         variables = self.selected_vars
-        self.info.set_output_summary(len(self.data[:, variables]),
-                                     format_summary_details(self.data[:, variables]))
         self.Outputs.reduced_data.send(self.data[:, variables])
 
         # Send the statistics of the selected variables to ouput
