@@ -10,8 +10,8 @@ import numpy as np
 import scipy.sparse as sp
 
 from AnyQt.QtWidgets import (
-    QGraphicsScene, QGraphicsView, QFormLayout, QComboBox, QGroupBox,
-    QMenu, QAction, QSizePolicy
+    QGraphicsView, QFormLayout, QComboBox, QGroupBox, QMenu, QAction,
+    QSizePolicy
 )
 from AnyQt.QtGui import QStandardItemModel, QStandardItem, QFont, QKeySequence
 from AnyQt.QtCore import Qt, QSize, QRectF, QObject
@@ -27,6 +27,7 @@ from Orange.widgets.utils import colorpalettes, apply_all, enum_get, itemmodels
 from Orange.widgets.utils.itemmodels import DomainModel
 from Orange.widgets.utils.stickygraphicsview import StickyGraphicsView
 from Orange.widgets.utils.graphicsview import GraphicsWidgetView
+from Orange.widgets.utils.graphicsscene import GraphicsScene
 from Orange.widgets.utils.colorpalettes import Palette
 
 from Orange.widgets.utils.annotated_data import (create_annotated_table,
@@ -444,7 +445,7 @@ class OWHeatMap(widget.OWWidget):
         gui.auto_send(self.buttonsArea, self, "auto_commit")
 
         # Scene with heatmap
-        class HeatmapScene(QGraphicsScene):
+        class HeatmapScene(GraphicsScene):
             widget: Optional[HeatmapGridWidget] = None
 
         self.scene = self.scene = HeatmapScene(parent=self)
@@ -667,7 +668,7 @@ class OWHeatMap(widget.OWWidget):
             self.selected_rows = self.__pending_selection
             self.__pending_selection = None
 
-        self.unconditional_commit()
+        self.commit.now()
 
     def __on_split_rows_activated(self):
         self.set_split_variable(self.row_split_cb.currentData(Qt.EditRole))
@@ -710,7 +711,7 @@ class OWHeatMap(widget.OWWidget):
         self.merge_indices = None
         if self.data is not None and self.merge_kmeans:
             self.update_heatmaps()
-            self.commit()
+            self.commit.deferred()
 
     def _make_parts(self, data, group_var=None, column_split_key=None):
         """
@@ -1055,11 +1056,11 @@ class OWHeatMap(widget.OWWidget):
 
     def __update_column_clustering(self):
         self.update_heatmaps()
-        self.commit()
+        self.commit.deferred()
 
     def __update_row_clustering(self):
         self.update_heatmaps()
-        self.commit()
+        self.commit.deferred()
 
     def update_legend(self):
         widget = self.scene.widget
@@ -1207,8 +1208,9 @@ class OWHeatMap(widget.OWWidget):
             self.selected_rows = list(self.scene.widget.selectedRows())
         else:
             self.selected_rows = []
-        self.commit()
+        self.commit.deferred()
 
+    @gui.deferred
     def commit(self):
         data = None
         indices = None
